#include <iostream>
#include <queue>
#include <vector>
#include <limits.h>
using namespace std;
typedef long long ll;
const ll maxn = 4e5+10, maxm = 8e5+10, maxt = 1e4+10, inf = 1e15, maxlog = 18;

struct node {
    ll x, cities, dist, parent, edge_dist;
    node(){}
    node(ll _x, ll _cities, ll _dist, ll _parent, ll _edge_dist) {
        x = _x;
        cities = _cities;
        dist = _dist;
        parent = _parent;
        edge_dist = _edge_dist;
    }
};

queue < node > q;
pair < ll, ll > path[maxn];
vector < pair < ll, ll > > v[maxn];
vector < pair < ll, ll > > tree[maxn];
ll sparse_lca[maxlog][maxn];
ll sparse_dist[maxlog][maxn];

void init_path() {
   q.push({1, -1, 0, 0, 0});
   path[1] = {-1, 0};
   while (!q.empty()) {
       node top = q.front();
       q.pop();
       if (top.dist > path[top.x].second) continue;
       tree[top.parent].push_back({top.x, top.edge_dist});
       tree[top.x].push_back({top.parent, top.edge_dist});
       sparse_lca[0][top.x] = top.parent;
       sparse_dist[0][top.x] = top.edge_dist;
       //cout << "edge: " << top.parent << " -> " << top.x << ' ' << top.edge_dist << '\n';
       //cout << "top: " << top.x << ' ' << top.cities << ' ' << top.dist << '\n';
       for (auto p : v[top.x]) {
            ll y = p.first;
            ll ed = p.second;
            if (path[y].first == -2 || (path[y].first == top.cities+1 && path[y].second > top.dist + ed)) {
                path[y] = {top.cities+1, top.dist + ed};
                q.push({y, top.cities+1, top.dist + ed, top.x, ed});
            }
       }
   }
}

ll n, m, queries;

void init_sparse() {
    for (int i = 1 ; i < maxlog ; ++i) {
        for (int j = 1 ; j <= n ; ++j) {
            sparse_lca[i][j] = sparse_lca[i-1][sparse_lca[i-1][j]];
            sparse_dist[i][j] = sparse_dist[i-1][j] + sparse_dist[i-1][sparse_lca[i-1][j]];
            //cout << "sparse: " << i << ' ' << j << ' ' << sparse_lca[i][j] << ' ' << sparse_dist[i][j] << '\n';
        }
    }
}

int depth[maxn];

void dfs(int x, int p) {
    depth[x] = depth[p] + 1;
    for (auto edge : tree[x]) {
        if (edge.first == p) continue;
        dfs(edge.first, x);
    }
}

node equalize(int x, int y) {
    //cout << "to equalize: " << x << ' ' << y << '\n';
    int cities = 0, dist = 0;
    for (int i = maxlog-1 ; i >= 0 ; --i) {
        //cout << "equalize: " << i << ' ' << x << ' ' << sparse_lca[i][x] << ' ' << sparse_dist[i][x] << ' ' << cities << ' ' << dist << '\n';
        if (depth[sparse_lca[i][x]] >= depth[y]) {
            cities += (1 << i);
            dist += sparse_dist[i][x];
            x = sparse_lca[i][x];
        }
    }
    //cout << "equalize end: " << x << ' ' << cities << ' ' << dist << '\n';
    return {x, cities, dist, 0, 0};
}

pair < int, int > lca(int x, int y) {
    if (depth[y] < depth[x]) swap(x, y);
    node res = equalize(y, x);
    if (res.x == x) return {res.cities, res.dist};
    return {-1, -1};
}

ll x, y, d;

int main () {
    ios_base :: sync_with_stdio(0);
    cin.tie(nullptr);
    cout.tie(nullptr);
    for (ll i = 0 ; i < maxn ; ++i)
        path[i] = {-2, inf};
    cin >> n >> m;
    for (ll i = 0 ; i < m ; ++i) {
        cin >> x >> y >> d;
        v[x].push_back({y, d});
        v[y].push_back({x, d});
    }
    init_path();
    init_sparse();
    dfs(1, 0);
    cout << path[n].first << ' ' << path[n].second << '\n';
    cin >> queries;
    for (int i = 1 ; i <= queries ; ++i) {
        cin >> x >> y;
        auto res = lca(x, y);
        if (res.first == -1) cout << 0 << '\n';
        else cout << res.first-1 << ' ' << res.second << '\n';
    }
    return 0;
}

/*
5 6
1 2 1
2 3 2
1 3 5
3 4 1
3 5 3
4 5 1
3
2 3
3 5
1 4

*/